import mock
from ixtest.mock import Mut, create_mock

from ixoncdkingress import types as sut

def test_response_code_status_line(monkeypatch):
    mut = Mut(sut.ResponseCode.status_line)
    mut.self = create_mock(name='self')

    mut.create_test_suite(monkeypatch)

    out_put = mut()

    assert f"{mut.self.value} {mut.self.name.replace.return_value}" == out_put

    assert [
        mock.call('_', ' ')
    ] == mut.self.name.replace.call_args_list



#!/usr/bin/env python
from setuptools import find_packages, setup

development_requires = [
    'bandit==1.7.4',
    'dominate==2.7.0',
    'ixtest==0.0.20',
    'mock==4.0.3',
    'mypy==0.971',
    'pyfakefs==4.6.3',
    'pylint==2.15.0',
    'pytest==7.1.2',
    'pytest-cov==3.0.0',
    'pytest-integration==0.2.2',
    'pytest-mock==3.8.2',
    'types-requests>=2.28.11.5',
    'wouter==0.0.3',
]

setup(
    name='ixoncdkingress',
    version='0.0.12',
    description='IXON CDK Ingress used in Custom Backend Components(CBC) for the IXON Cloud',
    author='IXON',
    author_email='development@ixon.cloud',
    url='https://www.ixon.cloud/',
    packages=find_packages(exclude=['tests*', ]),
    package_data={
        '': ['py.typed', 'assets/*'],
    },
    python_requires='>=3.9',
    install_requires=[
        'cryptography~=38.0.3',
        'docker>=6.1.3',
        'pymongo>=4.4.0',
        'requests>=2.28.1',
    ],
    extras_require={
        'development': development_requires,
    },
)



import mock, pytest

from ixtest.mock import Mut

from ixoncdkingress import __main__ as sut

def test_main_ok(monkeypatch):
    mut = Mut(sut.main)

    mut.create_test_suite(monkeypatch)

    out_put = mut()

    config = sut.Config.from_environ.return_value
    servlet = sut.Servlet.return_value

    assert 0 == out_put

    assert [
        mock.call(sut.os.environ)
    ] == sut.Config.from_environ.call_args_list

    assert [
        mock.call(sut.Config.from_environ.return_value)
    ] == sut.Servlet.call_args_list

    assert [
        mock.call(config, servlet)
    ] == sut.wsgi.run_server.call_args_list

def test_main_ki(monkeypatch):
    mut = Mut(sut.main)

    mut.create_test_suite(monkeypatch)

    sut.wsgi.run_server.side_effect = KeyboardInterrupt

    out_put = mut()

    assert 0 == out_put

def test_main_ie(monkeypatch):
    mut = Mut(sut.main)

    mut.create_test_suite(monkeypatch)

    sut.wsgi.run_server.side_effect = IndexError

    with pytest.raises(IndexError):
        out_put = mut()

        assert 0 == out_put



"""
Functions for finding and calling CBC functions.
"""
from typing import Any

import importlib
import sys

from ixoncdkingress.cbc.context import CbcContext
from ixoncdkingress.types import FunctionLocation, FunctionArguments
from ixoncdkingress.utils import handle_exception
from ixoncdkingress.webserver.config import Config
from ixoncdkingress.webserver.response import Response

def call_cbc( #pylint: disable=inconsistent-return-statements
        config: Config, context: CbcContext,
        function_location: FunctionLocation,
        function_kwargs: FunctionArguments,
        response: Response
    ) -> Any:
    """
    Finds, loads and calls the function specified in the body. The content_type specifies the
    format of the body.
    """

    # Get the specified module
    sys.path.insert(0, config.cbc_path)
    module = importlib.import_module(function_location[0])
    if config.production_mode is False:
        try:
            module = importlib.reload(module)
        except ImportError as error:
            handle_exception(error, response)
            return
    del sys.path[0]

    # Get the specified function
    if hasattr(module, function_location[1]):
        function = getattr(module, function_location[1])

        # Check if it's exposed
        if getattr(function, 'exposed', False):
            return function(context, **function_kwargs)

    return f"Not found, function '{'.'.join(function_location)}' does not exist or is not exposed"
